#include	<stdio.h>
#include        <signal.h>
#include	"FS.h"
#include	"FSlib.h"

#define TEST_FONT_NUM       1 
#define TEST_PATTERN       "*"
#define TIME_TO_WAIT        8 

#define OPEN_FAIL           11
#define SERVER_HANG         12
#define UNENOUGH_FONT       13
#define BAD_USAGE           -1

static char *servername = NULL;

static void usage(progname) 
char* progname;
{
    fprintf(stderr, "usage:  %s [-s server_name] [-n font_number] [-p pattern] [-t time_to_wait]\n", progname);
    fprintf(stderr, "  -s server_name   font server's name, e.g.tcp/aldns1:7001; default NULL.\n");
    fprintf(stderr, "  -n font_number   check if font_number of fonts are available. default 1.\n");
    fprintf(stderr, "  -p pattern       the pattern of the font(s), default \"*\".\n");
    fprintf(stderr, "  -t time_to_wait  after how many seconds if no answer the \"server hanging\" will be declared. default 8 seconds.\n");
    exit(BAD_USAGE);
}

/* time_out handler */
static void time_out(sig_no) 
int sig_no;
{
    printf("server %s is hanging.\n", servername);
    exit(SERVER_HANG);
}

void main(argc, argv)
int argc;
char **argv;
{
    FSServer           *svr;
    char               **fonts, 
                       *pattern=TEST_PATTERN;
    int                i, 
                       font_number=TEST_FONT_NUM, 
                       actualCount;
    unsigned           time_to_wait = TIME_TO_WAIT;
    struct sigaction   action;

    if(argc == 999 ) {
        /* this message is for strings(1). */
        printf("Here!!!.\n");
        printf("the source is in tian's pub/src. tian@aluxpo.micro.lucent.com, x7668. \n");
    }

    for(i=1; i<argc; i++) {
        char *arg = argv[i];
        if(arg[0] == '-') {
              switch (arg[1]) {
                 case 's': if(++i >= argc) usage(argv[0]);
                           servername = argv[i];
                           continue;
                 case 'n': if(++i >= argc) usage(argv[0]);
                           if( atoi(argv[i]) <= 1) {
                                fprintf(stderr, 
                                        "%s: time_to_wait must be an integer greater than 1.\n",
                                        argv[0]);
                                usage(argv[0]);
                           }
                           font_number = atoi(argv[i]);
                           continue;
                 case 'p': if(++i >= argc) usage(argv[0]);
                           pattern = argv[i];
                           continue;
                 case 't': if(++i >= argc) usage(argv[0]);
                           if( atoi(argv[i]) <=0 ) {
                                fprintf(stderr, 
                                        "%s: time_to_wait must be a positive integer.\n",
                                        argv[0]);
                                usage(argv[0]);
                           }
                           time_to_wait = (unsigned) atoi(argv[i]);
                           continue;
                 default : usage(argv[0]);
              }
        }
        else usage(argv[0]);
    }

    /* set the time_out before we try to connect the server */
    action.sa_handler = ( void(*)())time_out;
    sigemptyset(&action.sa_mask);
    if( sigaction( SIGALRM, &action, 0) == -1 ) {
        perror("signation");
        fprintf(stderr, "%s: failed at sigaction( SIGALRM, &action, 0).\n", argv[0]);
        exit(1);
    } 

    /* set the clock */
    (void)alarm(time_to_wait);
    
    /* connect the server, we may get blocked from here. */
    if( (svr=FSOpenServer(servername)) == NULL ) {
	printf("%s: unable to open server \"%s\".\n", 
               argv[0], servername? servername : "NULL");
	exit(OPEN_FAIL);
    }

    /* try to get some fonts */
    fonts = FSListFonts(svr, pattern, font_number, &actualCount);
    if(fonts == NULL || actualCount < font_number) {
	printf("%s: cannot get expected number of fonts. only get %d.\n", 
               argv[0], actualCount);
	exit(UNENOUGH_FONT);
    }
    FSFreeFontNames(fonts);
    
    printf("server %s is ok.\n", servername);

    /* sleep(4); */ /*to test the time_out()*/

    exit(0);
}
